"use strict";

module.exports = {
  "construction": require("./construction"),
  "tree-modification": require("./tree-modification"),
  "compare-document-position": require("./compare-document-position"),
  "named-properties": require("./named-properties"),
  "inner-html": require("./inner-html")
};
